import 'dart:async';
import 'dart:math';

import 'package:example/counter/counter.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

class CounterPage extends StatelessWidget {
  const CounterPage({super.key});

  @override
  Widget build(BuildContext context) {
    print('page');
    return BlocProvider(
      create: (_) => CounterCubit(),
      child: const CounterView(),
    );
  }
}

class Pair {
  final int x;
  final double y;

  const Pair({
    required this.x,
    required this.y,
  });

  @override
  String toString() {
    return 'Pair($x,${y.toStringAsFixed(2)})';
  }
}

class CounterView extends StatefulWidget {
  const CounterView({super.key});

  @override
  State<CounterView> createState() => _CounterViewState();
}

class _CounterViewState extends State<CounterView> {
  /// generate stream
  Stream<double> _test1() async* {
    final random = Random();
    var acc = 0;
    final now = DateTime.now();
    while (true) {
      for (final i in Iterable<int>.generate(9)) {
        yield random.nextDouble() * 4 - 2;
      }
      acc += 9;
      if (acc >= 20 * 250) {
        print(DateTime.now().difference(now));
        break;
      }
      await Future<void>.delayed(
        now.add(Duration(milliseconds: 4 * acc)).difference(DateTime.now()),
      );
      // await Future<void>.delayed(const Duration(milliseconds: 36));
    }
    // for (final i in Iterable<int>.generate(100)) {
    //   await Future<void>.delayed(const Duration(milliseconds: 36));
    //   yield i;
    // }
  }

  Stream<Pair> _test2() async* {
    final a = _test1();

    // final b = a.asBroadcastStream();
    // b.listen((event) { })

    var x = 0;

    await for (final i in a) {
      yield Pair(x: x, y: i);
      x += 1;
    }
  }

  Stream<List<Pair>> _test3() async* {
    print('_test3');
    final a = _test2();

    final res = <Pair>[];
    await for (final i in a) {
      if (res.length > 10000) res.removeRange(0, res.length - 7500);
      res.add(i);
      yield res;
    }
  }

  late final Stream<List<Pair>> _test4;

  @override
  void initState() {
    super.initState();

    _test4 = _test3();
  }

  @override
  Widget build(BuildContext context) {
    print('build');
    return Scaffold(
      body: StreamBuilder(
        stream: _test4,
        builder: (context, snapshot) {
          if (!snapshot.hasData) return const SizedBox();
          return Text(snapshot.data!.length.toString());
        },
      ),
    );
  }
}

class CounterText extends StatelessWidget {
  const CounterText({super.key});

  @override
  Widget build(BuildContext context) {
    final theme = Theme.of(context);
    final count = context.select((CounterCubit cubit) => cubit.state);
    return Text('$count', style: theme.textTheme.displayLarge);
  }
}
